from Numeric import *

def bindata(bindata, bins, datamin = None, datamax = None):
	"""Bins data for e.g. histograms

	A value is put in the bin when: binnr <= value < binnr+1
	Where value is scaled to the range [0, bins].

	This would mean that if bindata contains values in the range [0-N]
	and the min,max is 0,N, N will NOT be counted. 	e.g.:
	unbinned = [0,1,1,2,3,4,4]
	binned = bindata(unbinned, 4)
	binned contains [1,2,1,1]
	1+2+1+1 = 5, while our original unbinned data had 7 elements, 2 are lost.
	

	binnr:   0   1   2   3
	       |...|...|...|...|
	range: 0   1   2   3   4
	1x0 -> bin0
	2x1 -> bin1
	1x2 -> bin2
	1x3 -> bin3
	2x4 -> 4 >= datamax, the two fours won't be counted

	To solve this the last bin is extended, so that it contains values from
	binnr <= value <= binnr+1
	in stead of
	binnr <= value < binnr+1
	
	Now binned contains [3, 1, 1, 2]

	NOTE: this is essentially wrong, but it prevents losing data points.
	If you want to avoid this set datamax a bit smaller than max(bindata)
	for instance datamax=max(bindata) - 1e-6

	Arguments:
	bindata -- a sequence
	bins -- the number of bins that should be returned
	datamin -- if the value in bindata < datamin, it is not counted,
		default is min(bindata)
	datamax -- if the value in bindata >= datamax, it is not counted,
		default is max(bindata)
	"""
	
	if datamin == None:
		datamin = min(bindata)
	if datamax == None:
		datamax = max(bindata)

	length = len(bindata)
	#bindata = array(bindata)
	#scaled = (((bindata - datamin) / (float(datamax) - datamin)) * float(bins))#.astype(Int)
	#indices = searchsorted(sort(scaled), arange(bins))
	#end = len(scaled)
	#a = concatenate([indices, [end]])
	#print indices, end, a
	#return a[1:] - a[:-1]

	binData = zeros(bins)
	for i in xrange(0, length):
		binNo = int(math.floor( ((bindata[i] - datamin) / (float(datamax) - datamin)) * float(bins)))
		if binNo >= 0 and binNo < bins:
			binData[binNo] += 1
		elif binNo == bins:
			binData[binNo-1] += 1
	return binData
