"""Matrix class for 2d geometry"""
import types
from Numeric import *
import Numeric
import LinearAlgebra
import math
from vector import Vector
from tools import issequence
import time
from pdb import set_trace
class Matrix(object):
	def __init__(self, matrix=None):
		"""Constructor

		If no arguments are given it will construct an identity matrix

		Matrix is in row-column format

		m1 = Matrix()
		m1[0][2] = 10
		m1[1][2] = 20

		m2 = Matrix.translate((10, 20))

		m1 == m2

		Optional arguments:
		matrix -- a 3x3 sequence
		"""
		self.matrix = Numeric.array(matrix)
		if self.matrix == None:
			self.matrix = Numeric.identity(3, Numeric.Float)

	def inverse(self):
		"""Return the inverse of this matrix"""
		return Matrix(LinearAlgebra.inverse(self.matrix))

	def rotate(angle):
		"""Return a rotate matrix, angle in radians anticlockwise"""
		return Matrix(Numeric.array([[math.cos(angle), -math.sin(angle),0],[math.sin(angle), math.cos(angle),0], [0,0,1]], Numeric.Float))

	def translate(point):
		"""Return a translation matrix"""
		x, y = point
		return Matrix(Numeric.array([[1,0,x],[0,1,y], [0,0,1]], Numeric.Float))

	def scale(s):
		"""Returns a uniform scale matrix"""
		return Matrix(Numeric.array([[s,0,0],[0,s,0], [0,0,1]], Numeric.Float))

	def scaleXY(x, y):
		"""Returns a non-uniform scale matrix"""
		return Matrix(Numeric.array([[x,0,0],[0,y,0], [0,0,1]], Numeric.Float))

	def shear(x, y):
		"""Returns a shear matrix"""
		return Matrix(Numeric.array([[1,x,0],[y,1,0], [0,0,1]], Numeric.Float))

	def location(self):
		return Vector(self.matrix[0][2], self.matrix[1][2])

	def nolocation(self):
		matrix = Numeric.array(self.matrix)
		matrix[0][2] = 0
		matrix[1][2] = 0
		return Matrix(matrix)



	def __mul__(self, other):
		"""Multiplies with a Matrix or point"""
		#if type(other) == types.TupleType:
		if issequence(other) and len(other) == 2:
			x, y = other
			x, y = float(x), float(y)
			vec = array([x, y, 1])
			newvec = Numeric.matrixmultiply(self.matrix, vec)
			newx = newvec[0] / newvec[2]
			newy = newvec[1] / newvec[2]
			return Vector((newx, newy))
		elif type(other) == Matrix:
			newmatrix = Numeric.matrixmultiply(self.matrix, other.matrix)
			return Matrix(newmatrix)
		else:
			raise Exception, "can only multiply with 'point tuples', and Matrices"

	def mulXY(self, x, y):
		"""Muliplies the matrix with each x/y element

		Arguments:
		x -- a sequence with x elements
		y -- a sequence with y elements
		"""
		start = time.time()
		length = min(len(x), len(y))
		maxlength = max(len(x), len(y))
		newx = Numeric.zeros(length, Float)
		newy = Numeric.zeros(length, Float)
		x = x[:length]
		y = y[:length]
		w = Numeric.ones(length, Float)
		points = transpose(array([x,y,w]))
		#print x, y
		for i in xrange(length):
			point = Numeric.matrixmultiply(self.matrix, points[i])
			newx[i] = point[0] / point[2]
			newy[i] = point[1] / point[2]
		end = time.time()
		#if end - start > 1.0:
		#	set_trace()
		return newx,newy

	def __eq__(self, other):
		"""Return self.equals(other)"""
		return self.equals(other)

	def __ne__(self, other):
		"""Return not self.equals(other)"""
		return not self.equals(other)

	def equals(self, other, sigma=1e-6):
		"""Compares 2 matrices

		Return false if the difference between 2 corresponding elements
		is larger than sigma
		"""
		if other == None:
			return 0
		error = max(abs(ravel(self.matrix) - ravel(other.matrix)))
		return error <= sigma

	def __repr__(self):
		"""Nice representation of the matrix"""
		return \
			self.__class__.__module__ +"." +\
			self.__class__.__name__ +"<" +str(self.matrix) +">"


	def __len__(self):
		"""Length is always 3, (3 rows)"""
		return 3

	def __getitem__(self, index):
		"""Return row"""
		return self.matrix[index]

	def __setitem__(self, index, value):
		"""Set row"""
		self.matrix[index] = value


	rotate = staticmethod(rotate)
	translate = staticmethod(translate)
	scale = staticmethod(scale)
	scaleXY = staticmethod(scaleXY)
	shear = staticmethod(shear)

