"""For gridlines, at x and y values"""
import Kplot
from nonbasic import Line
from plotobject import PlotObject
from properties import *
from axis import _AxisLine
from basic import Polyline
from Numeric import *

def subdiv(p1, p2):
	n = 10
	x = zeros(n) + 0.0
	y = zeros(n) + 0.0
	v = p2 - p1
	for i in xrange(n):
		p = p1 + v.scale(float(i)/(n-1))
		x[i] = p.x
		y[i] = p.y
	return (x, y)
		
		
def transform(point, xtrans, ytrans):
	#print point
	x = xtrans.transform(point.x)
	y = ytrans.transform(point.y)
	return Kplot.Vector((x,y))


class Grid(PlotObject):
	"""Draws gridlines, usually used in combination with axes.
	vertical gridlines are drawn for each value in xticks, and
	horizontal for yticks.

	Properties:
	- xticks
	- yticks
	- color
	- linewidth
	- linestyle
	"""
	xticks = autoproperty(default=[])
	yticks = autoproperty(default=[])
	color = colorproperty()
	linewidth = linewidthproperty()
	linestyle = linestyleproperty()

	def __init__(self, xticks, yticks, defaults=None, **properties):
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.xticks = xticks
		self.yticks = yticks

	def plot(self, dev):
		xticks = self.xticks
		yticks = self.yticks

		matrix = (dev.getWorldMatrix()).inverse()
		bottomleft = matrix * (0, 0)
		bottomright = matrix * (1, 0)
		topright = matrix * (1, 1)
		topleft = matrix * (0, 1)

		bottom = _AxisLine(bottomleft, bottomright)
		top = _AxisLine(topleft, topright)
		left = _AxisLine(bottomleft, topleft)
		right = _AxisLine(bottomright, topright)

		sides = [bottom, top, left, right]
		done = []
		for side in sides:
			done.append(side)
			others = [i for i in sides if i not in done]
			for other in others:
				for tick in xticks:
					p1 = side.xtick(tick)
					p2 = other.xtick(tick)
					if p1 and p2:
						x, y = subdiv(p1, p2)
						line = Polyline(x,y, color=self.color, linestyle=self.linestyle, linewidth=self.linewidth)
						line.plot(dev)
				for tick in yticks:
					p1 = side.ytick(tick)
					p2 = other.ytick(tick)
					if p1 and p2:
						x, y = subdiv(p1, p2)
						line = Polyline(x,y, color=self.color, linestyle=self.linestyle, linewidth=self.linewidth)
						line.plot(dev)



	def example():
		import Kplot

		print "Grid example"
		print
		print 'Grid draws a grid from a range of x coordinates'
		print 'and a set of y coordinates:'
		print 'grid = Kplot.Grid(xticks=range(1,6), yticks=range(25,76, 25))'
		grid = Kplot.Grid(xticks=range(1,6), yticks=range(25,76, 25))
		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(grid,Kplot.Box())
		panel.setWorld(p1=(0, 0), p2=(6, 100))
		dev.output()
		dev.close()

	example = staticmethod(example)





