"""A label outside the viewport e.g. for labeling a graph"""
from plotobject import PlotObject
from properties import *
from autoproperty import *
from basic import Text
from Kplot import toradians
import Kplot

class Label(PlotObject):
	"""Plotobjects which labels panels.

	Input is two strings. The first is the string which needs to be plotted,
	the second is the position:
	- top
	- bottom
	- left
	- right

	Properties:
        - text
        - side
        - alignment
        - displacement
        - units
	"""
	text = autoproperty(typelist=[Text])
	alignment = stringproperty(default="center")
	side = stringproperty()
	displacement = numberproperty(default=2)
	units = unitsproperty()


	def __init__(self, text, side, defaults=None, **properties):
		PlotObject.__init__(self, defaults=defaults, properties=properties)
		self.text = Text(text, (0,0), defaults=properties)
		self.text.valign = "bottom"
		self.side = side
		self.text.addChangeListener(self)

	def plot(self, dev):
		x, y = 0, 0
		clipping = dev.getClipping()
		dev.setClipping(False)
		if self.side == "top":
			y = 1
			direction = Kplot.Vector(0, 1)
			self.text.textangle = 0
			self.text.valign = "bottom"
		elif self.side == "bottom":
			y = 0
			direction = Kplot.Vector(0, -1)
			self.text.textangle = 0
			self.text.valign = "top"
		elif self.side == "left":
			x = 0
			direction = Kplot.Vector(-1, 0)
			self.text.textangle = toradians(90)
			self.text.valign = "bottom"
			halign = {"bottom":"left", "center":"center", "top":"right"}
			self.text.halign = halign[self.alignment]
		elif self.side == "right":
			x = 1
			direction = Kplot.Vector(1, 0)
			self.text.textangle = toradians(-90)
			self.text.valign = "bottom"
			halign = {"bottom":"right", "center":"center", "top":"left"}
			self.text.halign = halign[self.alignment]
		else:
			raise Exception, "illigal side " +str(self.side)

		if self.side in ["top", "bottom"]:
			if self.alignment == "left":
				x = 0
				self.text.halign = "left"
			elif self.alignment == "center":
				x = 0.5
				self.text.halign = "center"
			elif self.alignment == "right":
				x = 1
				self.text.halign = "right"
			else:
				raise Exception, "illigal alignment " +str(self.alignment)

		if self.side in ["left", "right"]:
			alignments = {"bottom":0, "center":0.5, "top":1}
			try:
				y = alignments[self.alignment]
			except KeyError:
				raise Exception, "illigal alignment " +str(self.alignment)

		ref = Kplot.Vector(x, y)

		unitsmatrix = dev.getUnitMatrix(self.units)
		invunitsmatrix = unitsmatrix.inverse()

		worldmatrix = dev.getWorldMatrix()
		invworldmatrix = worldmatrix.inverse()

		location = (invunitsmatrix * ref) + direction.scale(self.displacement)
		location = unitsmatrix * location
		dev.pushWorldMatrix(invworldmatrix)
		self.text.location = location
		clipping = dev.getClipping()
		self.text.plot(dev)
		dev.setClipping(clipping)
		dev.popWorldMatrix()
		dev.setClipping(True)

	def example():
		import Kplot

		print "Label example"
		print
		print 'Label writes labels on the outside of a panel.'
		print 
		print 'Input is two strings. The first is the string which needs to be plotted,'
		print 'the second is the position:'
		print '- top'
		print '- bottom'
		print '- left'
		print '- right'
		print
		print 'A few examples:'
		print 'label1 = Kplot.Label("top label, centered", "top", alignment="center")'
		print 'label2 = Kplot.Label("left label, centered", "left", alignment="center")'
		print 'label3 = Kplot.Label("bottom label, right aligned", "bottom", alignment="right")'
		print 'label4 = Kplot.Label("right label, top aligned", "right", alignment="top")'
		label1 = Kplot.Label("top label, centered", "top", alignment="center")
		label2 = Kplot.Label("left label, centered", "left", alignment="center")
		label3 = Kplot.Label("bottom label, right aligned", "bottom", alignment="right")
		label4 = Kplot.Label("right label, top aligned", "right", alignment="top")

		dev = Kplot.createDevice()
		panel = dev.createPanel()
		panel.add(label1, label2, label3, label4)
		dev.output()
		dev.close()
		print


	example = staticmethod(example)
