import types
import Kplot
import Numeric
import os
from psbasedevice import PsBaseDevice

class GsDevice(PsBaseDevice):
	def __init__(self, size=(200, 200), units=None, device="", filename="", cmdlineargs="", promptOnClose=True):
		self.cmdlineargs = cmdlineargs
		self.promptOnClose = promptOnClose
		self.device = device
		self.filename = filename
		PsBaseDevice.__init__(self, size=size, units=units)


	def open(self):
		PsBaseDevice.open(self)
		cmd = os.environ.get("GS_BINARY", "gs")
		s = "%s %s -dNOPAUSE -dNOPROMPT -r%i -g%ix%i" % (cmd, self.cmdlineargs, self.dpi, self.res.x, self.res.y)
		if self.device:
			s += " -sDEVICE=%s" % self.device
		if self.filename:
			s += " -sOutputFile=%s " % self.filename
		s += " " +os.environ.get("KPLOT_GS_CMDLINE", "")
		s += " -" # read from stdin
		self.outpipe, self.inpipe = os.popen2(s)
		self.outputHeader()
		self.outputHeaderCode()
		self.setDimensions()


	def write(self, code):
		#print code,
		try:
			self.outpipe.write(code)
		except:
			print "GS output:"
			print self.inpipe.read()
			raise Exception, "GS error"

	def writeln(self, code):
		#print code
		try:
			self.outpipe.write(code+"\n")
		except:
			print "GS output:"
			print self.inpipe.read()
			raise Exception, "GS error"

	def flush(self):
		try:
			self.writeln("prompt")
			self.outpipe.flush()
		except:
			print "GS output:"
			print self.inpipe.read()
			raise Exception, "GS error"

	def clear(self):
		self.writeln("showpage");
		self.flush();
		PsBaseDevice.clear(self)

	def close(self):
		self.outpipe.flush()
		if self.promptOnClose:
			raw_input("press enter");
		self.writeln("showpage")
		self.writeln("quit")
		self.flush()

	def printOutput(self):
		print self.inpipe.read()

class WindowDevice(GsDevice):
	def __init__(self, size=(200, 200), units=None):
		GsDevice.__init__(self, size=size, units=units)

class PngDevice(GsDevice):
	def __init__(self, filename=None, size=(200, 200), units=None, cmdlineargs=""):
		if filename == None:
			filename = os.environ.get("KPLOT_FILENAME", "Kplot")
		GsDevice.__init__(self, size=size, units=units, filename=filename+".png",
			device="png16m", cmdlineargs=cmdlineargs, promptOnClose=False)

	def clear(self):
		prompt = self.promptOnClose
		self.promptOnClose = False
		self.close()
		self.open()
		self.promptOnClose = prompt

class NullDevice(GsDevice):
	def __init__(self, size=(200, 200), units=None):
		GsDevice.__init__(self, device="nullpage", size=size, units=units)
		self.promptOnClose = False



