import Kplot

class Graph(object):
	"""Plots a simple x/y graph

	example:
		import Kplot
		g = Kplot.simple.Graph(x=[0,1,2,3], y=[3,4,7,2])

	properties of PlotObjects can be changed, and the plot will repaint itself.

	Attributes of Graph:
		device:      a Kplot.Device object
		panel:       a Kplot.Panel object
		line:        the graph itself, a Kplot.Polyline object
		box:         the grid and axes, a Kplot.Box object
		toplabel:
		leftlabel:
		bottomlabel:
		rightlabel: the four labels around the graph, bottomlabel has the text
		            "x", the leftlabel the text "y"

	example:
		g.toplabel.text.text = "f(x) = x ** 2"
		g.toplabel.text.color = "red"
		g.box.xinterval = 1
		g.box.drawgrid = False

	g.toplabel.text.text may look strange, but g.toplabel.text is a Kplot.Text
	object, where text is one of it's properties.

	To find information about a PlotObject, use <object>.printInfo()
	or use the help function. The help function will give a description of the
	class.

	example:
		g.box.printInfo()
		help(Kplot.Box)



	"""
	def __init__(self, x, y, p1=None,p2=None):
		self.device = Kplot.createDevice()
		self.device.setAutoReplot(False)
		
		self.panel = self.device.createPanel()
		self.line = Kplot.Polyline(x,y)
		self.box = Kplot.Box()
		self.toplabel = Kplot.Label("", "top")
		self.leftlabel = Kplot.Label("y", "left", displacement=15)
		self.bottomlabel = Kplot.Label("x", "bottom", displacement=15)
		self.rightlabel = Kplot.Label("", "right")

		self.panel.add(self.box)
		self.panel.add(self.line)
		self.panel.add(self.toplabel, self.bottomlabel, self.leftlabel, self.rightlabel)
		self.setWorld(p1,p2)
		self.device.setAutoReplot(True)

	def __del__(self):
		self.device.close()

	def setWorld(self,p1,p2):
		if p1 and p2:
			self.panel.setWorld(p1,p2)
		else:
			ymargin = 0.1 * (max(self.line.y)-min(self.line.y))
			xmargin = 0.1 * (max(self.line.x)-min(self.line.x))
			self.panel.setWorldXY(self.line.x, self.line.y,xmargin=xmargin,ymargin=ymargin)
		self.replot()

	def replot(self):
		self.device.output()
