"""
-Normal interface
	-Device
		-What is a Device
			[It acts like a surface to plot on, does primitive plot operations]
			[!normalised space!]
		-How do i use a Device
			[Plot polylines, polygons, example]
		-What devices are there
			[Pgplot, Postscript, Qt?]
		-Device details
			[World matrix, Viewport matrix]
	-PlotObject
		-What is a PlotObject
			[It contains algorithms to plot more complex objects, provides defaults]
		-How do i use a  PlotObject
			[Plot complex objects, Rectangle example]
		-Making your own PlotObject
	-Panel
		-What is a Panel
			[A container for plotobjects, holds the state of the whole plot,
				ready to replot on changes]
		-How do i use a Panel
			[setWorld(XY/Image), example, split, mosaic, example]
	-An overview of Device, PlotObject and Panel
"""
import os

chapter("Normal interface",0)
text("""The classes described in this chapter make it possible to make complex
plots. There are 3 concepts each explained in a seperate paragraph.
The first concept is a <i>Device</i>.
A Device is responsible for doing the actual drawing. It can be a
pixel based output window on your screen, or a postscript file. Then there are
<i>PlotObjects</i>, these include drawing more complex things like e.g a histogram.
At last there is the panel. It acts like a container for one or more PlotObjects.
Each Panel has its own coordinate space and location on the Device.
""");

chapter("Device",1)
chapter("What is a Device", 2)
text("""A Device is a really primitive class for basic plotting
operations. It can draw polylines, polygons, circles, text and images.
These operations should be enough to construct more complex objects.
A Device can be a window on your screen, or a file, but they can all do the
same plotting operations.
""");
chapter("How do i use a Device", 2)
text("""The Kplot.createDevice function creates a default Device for you.
The Kplot default is an available Device on your system which displays in
a window.
In the next paragraph I will show your how to change this default device.
""")

code("""Python 2.2.1 (#1, Apr 11 2002, 15:27:15)
[GCC 2.95.3 20010315 (release)] on linux2
Type "help", "copyright", "credits" or "license" for more information.
>>> import Kplot
>>> dev = Kplot.createDevice()
>>> dev.plotPolyline(x=[0,0.5, 1], y=[0, 0.2, 0.9], color="red", linewidth=1, linestyle="normal")
>>> dev.plotPolygon(x=[0.5, 0.9, 0.1], y=[0.1, 0.5, 0.5], color="blue", linewidth=1, linestyle="normal", fillstyle="solid")
""")
text("""The first line imports the Kplot package. The second line creates the
default Device. The last two lines show how to use a Device. It does really
primitive plot operations, and requires alot of arguments. The output, probably on
a window, should look somewhat like the next picture""")
os.environ["KPLOT_FILENAME"] = "device1"
os.system("cd images; python device1.py")
image("images/device1.png")
text("""The Kplot.createDevice function creates a default Device. Another way to
create a Device is by calling the constructor of the class directly.
The same arguments given to Kplot.createDevice can be passed to the constructors
of the Device classes. The Kplot.ps.WindowDevice and Kplot.pgplot.WindowDevice are two
examples of such classes.""")
code("""Python 2.2.1 (#1, Apr 11 2002, 15:27:15)
[GCC 2.95.3 20010315 (release)] on linux2
Type "help", "copyright", "credits" or "license" for more information.
>>> import Kplot
>>> dev1 = Kplot.ps.WindowDevice()
>>> dev2 = Kplot.pgplot.WindowDevice()""")
note("""Kplot isn't limited to a single device, you can create as many as you
like in the same session/script, even of the same type. So it's also possible to
open two pgplot devices.""")

chapter("""What devices are there""", 2)

text("""Usable devices are the PGPLOT and Postscript/Ghostscript
Devices. They can both output to a window, PostScript file, or an image.
Some preparation has been done to make a Device using the Qt toolkit, and
might be added in the future.""")

text("""To change the default device(the device created with the Kplot.createDevice() function)
for Kplot, you can set the KPLOT_DEVICE environmental variable.
This can be set to the full class name, Kplot.ps.WindowDevice and Kplot.pgplot.WindowDevice
being two examples.""")

code("""
unix csh:
$ setenv KPLOT_DEVICE Kplot.ps.WindowDevice
unix bash:
$ export KPLOT_DEVICE=Kplot.ps.WindowDevice
windows:
c:\>set KPLOT_DEVICE=Kplot.ps.WindowDevice
""")

note("""The environmental variable can also be set from within python, using the
os module. os.environ["KPLOT_DEVICE"] can be assigned a new string, and Kplot.createDevice
will use that as default for the next call.""")

text("""There also is a Kplot.createHardcopyDevice function. This should return a Device
for making a hardcopy, a file. The default can be changed using the KPLOT_HARDCOPY_DEVICE
environmental variable, simular to KPLOT_DEVICE. See chapter 'Devices' for a list of devices
and the details about them.
""")



"""Kplot.ps.PngDevice creates .png files, this is a graphics format
free of any patents, unlike gif and jpeg."""

"""When a Device makes a hardcopy, it always has a
default name, Kplot.<suffix>, where suffix can be ps, png etc.
The filename can also be given as keyword argument
to the Kplot.createDevice function or the constructor of the Device class.
If the KPLOT_FILENAME is set, and no filename is given, it is used instead."""

"""Don't give the filename a suffix, like plot.png or plot.ps.
Depending on the output format of the device, a suffix will be given, so
'filename=plot' will result in the file plot.png for a Kplot.ps.PngDevice
"""



chapter("Device details", 2)
text("""
TODO: World/Viewport (Matrix)
""")

# ---------------- PlotObjects ----------------
chapter("PlotObject",1)
text("""Making plots with many primitives operations wouldn't be
very efficient. It is very flexible, because almost everything is
possible, but it will take much work to make complex plots. And
alot of duplicate code to solve the same problem over and over again.
That is where the PlotObject comes in.
""")

chapter("What is a PlotObject", 2)
text("""
A PlotObject contains algorithms to plot more complex objects. It also
provides default values, for attributes like color, linewidth etc.
All PlotObjects are in the Kplot.plotobjects package, but they are also
imported into the Kplot package itself to avoid alot of typing. Meaning
that Kplot.plotobjects.Line and Kplot.Line are both the same Line class
""")

chapter("How do i use a  PlotObject", 2)
text("""
I will show you how to use two simple PlotObjects, Rectangle and Line.
You can create a Rectangle by calling it's constructor. As your can
read in the API documentation, it takes two arguments, two points
forming a rectangle. Note that calling the constructor means that
you indirectly call the __init__ method.""")
code("""
>>> help(Kplot.Rectangle)

Help on class Rectangle in module Kplot.plotobjects.nonbasic:

class Rectangle(Kplot.plotobjects.plotobject.PlotObject)
 |  Draws a single rectangle
 |
 |  Method resolution order:
 |      Rectangle
 |      Kplot.plotobjects.plotobject.PlotObject
 |      __builtin__.object
 |
 |  Methods defined here:
 |
 |  __init__(self, p1, p2, defaults=None, **properties)
 |      Constructor
 |
 |      Arguments:
 |      p1 -- the lower left corner of the rectangle
 |      p2 -- the upper right corner of the rectangle
 |
 |  plot(self, dev)
 |
.........
""")
text("""Every PlotObject has a plot method taking a Device as argument.
A PlotObject can use the Device to draw a more complex object by calling
the primitive operation as needed.""")
code("""Python 2.2.1 (#1, Apr 11 2002, 15:27:15)
[GCC 2.95.3 20010315 (release)] on linux2
Type "help", "copyright", "credits" or "license" for more information.
>>> import Kplot
>>> rect = Kplot.Rectangle(p1=(0.1, 0.1), p2=(0.5, 0.8), color="green")
>>> line = Kplot.Line(p1=(0,0), p2=(1,1), color="red")
>>> dev = Kplot.createDevice()
>>> rect.plot(dev)
>>> line.plot(dev)""")
text("""As you can see you can create PlotObjects before you create a Device.
Your output should look somewhat like the next picture.""")
os.environ["KPLOT_FILENAME"] = "plotobject1"
os.system("cd images; python plotobject1.py")
image("images/plotobject1.png")

chapter("Making your own PlotObject", 2)
text("""Making your own PlotObject involves subclassing Kplot.PlotObject,
adding a suitable constructor and implementing the plot method.
""")

from string import join
import os
codestr = join(file("images/plotobject2.py").readlines())
codestr = codestr.replace(">", "&gt;")
codestr = codestr.replace("<", "&lt;")
code(codestr)
text("""Your output should look somewhat like the next picture.""")
os.environ["KPLOT_FILENAME"] = "plotobject2"
os.system("cd images; python plotobject2.py")
image("images/plotobject2.png")

chapter("Panel",1)
text("""With complex plots, you'll soon end up having alot of PlotObjects.
A change to one of the properties of a PlotObject and you have to manually
replot all PlotObjects on the Device, not ideal for the interactive Python
shell. On top of that, you probably want to have several 'regions' on your
plot, with their own world space coordinates.""")

chapter("What is a Panel", 2)
text("""
A Panel is a container for PlotObjects. PlotObjects can be added to and removed
from a Panel. It has it's own viewport, a region on the Device. And it has
it's own world space coordinates.""")

chapter("How do i use a Panel", 2)
text("""A Panel can be made by calling Device.createPanel. Without arguments
a default region of the Device is used. It can take two arguments p1 and p2
the lower left point, and the upper right point, in normalised coordinates
([0..1] domain). The default for p1 is (0.1, 0.1), and for p2 it is (0.9, 0.9).
""")
code("""Python 2.2.1 (#1, Apr 11 2002, 15:27:15)
[GCC 2.95.3 20010315 (release)] on linux2
Type "help", "copyright", "credits" or "license" for more information.
>>> import Kplot
>>>
>>> rect = Kplot.Rectangle(p1=(1, 2), p2=(4, 8), color="green")
>>> line = Kplot.Line(p1=(2,5), p2=(5,1), color="red")
>>> box = Kplot.Box()
>>> dev = Kplot.createDevice()
>>> panel = dev.createPanel()
>>> panel.setWorld(p1=(0,0), p2=(6,10))
>>> panel.add(box, rect, line)
>>> dev.output()""")

text("""Your output should look somewhat like the next picture.""")
os.environ["KPLOT_FILENAME"] = "panel1"
os.system("cd images; python panel1.py")
image("images/panel1.png")

note("""As you can see the world coordinates are set using panel.setWorld.""")
text("""There is a usefull feature which can be really usefull when you are
in the interactive python shell. When you change an attribute
of a PlotObject, Kplot will call Device.output to redraw everything.
By calling dev.setAutoReplot you can enable and disable the automatic replotting.
For the interactive shell it is enabled by default, otherwise (from a script)
is is disabled. This is because scripts run in batches, and only need to
display the last result and from the interactive shell you want to see every
change immediatly. Continuing from the last example, we will continue""")
code("""
>>> line.color = "yellow"
>>> box.xinterval = 2
>>> box.leftaxis.textangle = Kplot.toradians(90)
>>> rect.p2 = (5,9)
>>> dev.setAutoReplot(False)
>>> rect.color = "red"
>>> dev.output()
""")
text("""Notice that after each change in one of the attributes, everything
gets redrawn. By calling dev.setAutoReplot(False) this behaviour stops.
""")
text("""TODO: setWorld(XY/Image), example, split, mosaic, example""")
chapter("Making hardcopies", 1)
text("""At any time, when you have a plot, you can create a hardcopy. Assuming
you are using Panel and PlotObjects to construct your plot. By calling Device.hardcopy
with an optional filename as argument, a hardcopy is produced using the default hardcopy
Device. The hardcopy Device is created using the Kplot.createHardcopyDevice function.
To change the default hardcopy device, you can set the KPLOT_HARDCOPY_DEVICE environmental
variable the same way as you did the KPLOT_DEVICE variable.
KPLOT_HARDCOPY_SIZE
""")
chapter("An overview of Device, PlotObject and Panel", 1)
text("""TODO""")



