from reportlab.platypus import SimpleDocTemplate, Paragraph, \
	Spacer, XPreformatted, Flowable, PageBreak, Image
from reportlab.platypus.xpreformatted import PythonPreformatted
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.rl_config import defaultPageSize
from reportlab.lib.units import inch, mm
from reportlab.platypus.paragraph import _leftDrawParaLine, setXPos
import reportlab
from string import join
import re


PAGE_HEIGHT=defaultPageSize[1];
PAGE_WIDTH=defaultPageSize[0]
styles = getSampleStyleSheet()
Title = "Kplot - Object Oriented vector plotting for Python"
pageinfo = Title
id = 0


def makeAnchor(text):
	return text.replace(" ", "_") + "_anchor"

class Bookmark(Paragraph):
	def __init__(self, title, depth):
		Paragraph.__init__(self, "", styles["Normal"])
		#Paragraph.__init__(self,"", styles["Normal"])
		self.title = title
		self.depth = depth
	#def wrap(self, *args):
	#	return (1.0,2.0)
	def drawPara(self,debug=0):
		Paragraph.drawPara(self,debug)
		#global id
		#Flowable.drawPara(self, debug)
		anchor = makeAnchor(self.title)
		#self.canv.bookmarkPage(anchor)
		x, y = self.canv._currentMatrix[4:6]
		y += inch
		self.canv.bookmarkHorizontalAbsolute(anchor, 0)
		self.canv._destinations[anchor].xyz(x,y,0)
		self.canv._destinations[anchor].fmt.left = x
        #result = d[name] = pdfdoc.Destination(name) # newly defined, unbound
		self.canv.addOutlineEntry(self.title, anchor, self.depth,1)
		self.canv.showOutline()
		#print "blaat"

class LinkParagraph(Paragraph):
	def __init__(self, text, style):
		Paragraph.__init__(self, text, style)
		self.wordlist = []

	def drawPara(self,debug=0):
		Paragraph.drawPara(self,debug)
		lines = self.blPara.lines
		if self.blPara.kind != 0:
			#line = join(line[1])
			lines = [x.words[0].text for x in lines]
		else:
			lines = [join(x[1]) for x in lines]
		print lines

		nLines = len(lines)

		#set_trace()
		#print lines[0].words[0].text
		print lines[0][1]

		for line, linenr in zip(lines, range(nLines)):
			#line = join(line[1])
			print "line:", line
			for word in self.wordlist:
				print word
				offset = line.find(word)
				if offset != -1:
					x1 = self.canv.stringWidth(line[:offset],self.style.fontName, self.style.fontSize)
					x2 = x1 + self.canv.stringWidth(word,self.style.fontName, self.style.fontSize)
					y1 = self.height - self.style.leading * linenr
					y2 = y1 - self.style.leading
					rect = (x1, y2, x2, y1)
					#self.canv.linkRect("dsadsa", makeAnchor(word), rect)#, Border='[10,0,21]')
					self.canv.linkAbsolute("dsadsa", makeAnchor(word), rect)
					print "rect:", rect

		#width, height = self.wrap(self.minWidth(), 1000000)
		#print width, height
		#self.canv.linkAbsolute(self.text, makeAnchor(self.text),
		#global id
		#Flowable.drawPara(self, debug)
		#self.canv.bookmarkPage("para#" + str(id))
		#self.canv.addOutlineEntry(self.title, "para#" + str(id), self.depth,1)
		#self.canv.showOutline()
		#id += 1
		#print "blaat"
		#set_trace()


def myFirstPage(canvas, doc):
	def testfunc(a, b, c):
		print a, b, c
	canvas.blaat1 = testfunc
	canvas.saveState()
	canvas.setFont('Times-Bold',16)
	canvas.drawCentredString(PAGE_WIDTH/2.0, PAGE_HEIGHT-108, Title)
	canvas.setFont('Times-Roman',9)
	canvas.drawString(inch, 0.75 * inch, "First Page / %s" % pageinfo)
	canvas.restoreState()

def myLaterPages(canvas, doc):
	canvas.saveState()
	canvas.setFont('Times-Roman',9)
	canvas.drawString(inch, 0.75 * inch, "Page %d %s" % (doc.page, pageinfo))
	canvas.restoreState()

class OutputPDF:
	def __init__(self, name, wordlist):
		self.wordlist = wordlist
		self.doc = SimpleDocTemplate(name +".pdf")
		self.stories = [Spacer(1,1*inch)]
		styles = getSampleStyleSheet()
		#self.textStyle = styles["Normal"]
		self.textStyle = ParagraphStyle("normal", fontName = "Times-Roman", leftIndent=inch*0.25, firstLineIndent=0.0*inch)
		self.codeStyle = ParagraphStyle("code", fontName = "Courier-bold", leftIndent=inch*0.50)
		self.noteStyle = ParagraphStyle("note", fontName = "Times-Roman", leftIndent=inch*0.50, firstLineIndent=0.0*inch)
		#self.stories.append(PageBreak())

		self.headingStyles = []
		self.headingStyles.append(styles["Heading1"])
		self.headingStyles.append(styles["Heading2"])
		self.headingStyles.append(styles["Heading2"])
		self.headingStyles.append(styles["Heading2"])
		self.headingStyles.append(styles["Heading2"])
		self.headingStyles.append(styles["Heading2"])
		self.headingStyles.append(styles["Heading2"])
		#self.headingStyle[0] = styles["Heading1"]
		#self.codeStyle = ParagraphStyle("dsa", fontName = "Helvetica", leftIndent=inch*0.5)
		#print styles

	def close(self):
		self.doc.build(self.stories, onFirstPage=myFirstPage, onLaterPages=myLaterPages)

	def chapter(self, title, depth=0):
		if(depth == 0):
			self.stories.append(PageBreak())
		p = Paragraph(title, self.headingStyles[depth])
		self.stories.append(p)
		self.stories.append(Bookmark(title, depth))
		if depth != 0:
			self.stories.append(Spacer(1,0.2*inch))

	def addImage(self, filename, width=None, height=None):
		self.stories.append(Spacer(1,0.2*inch))
		if width != None: width *= mm
		if height != None: height *= mm

		self.stories.append(Image(filename, width=width, height=height))
		self.stories.append(Spacer(1,0.2*inch))

	def addText(self, text):
		#for word in self.wordlist:
		#	text = text.replace(word, "<b>" +word +"</b>")
		#print text
		def addCleanText(text):
			#offset = -1;
			#for word in wordlist:
			#	place = text.find(word)
			#	if place != -1
			p = Paragraph(text, self.textStyle)#, self.wordlist)
			self.stories.append(p)


		def addLink(name):
			p = LinkParagraph(name, name, self.textStyle)
			self.stories.append(p)
			print self.textStyle
			pass

		def _addText(text):
			l = [text.find(x) for x in self.wordlist]
			if l:
				offset = min(l)
				word = self.wordlist[l.index(offset)]
				addCleanText(text[:offset])
				addLink(word)
				addCleanText(text[offset+len(word):])
			else:
				self.addCleanText(text)

		addCleanText(text)
		#_addText(text)
		#global drawPara
		#rest = text
		#reduce(lambda x,y:
		#for word in self.wordlist:
		#	offset = rest.find(
		#	while len(rest) > 0:
		#	p = Paragraph(rest, self.textStyle)
		#	self.stories.append(p)
		#	self.stories.append(LinkParagraph("blat", text, self.textStyle))
		self.stories.append(Spacer(1,0.2*inch))
	def addNote(self, text):
		p = Paragraph("<i>" +text +"</i>", self.noteStyle)#, self.wordlist)
		self.stories.append(p)

	def addCode(self, text):
		text = re.sub("\t", " " * 4, text)
		text = text.replace(">", "&gt;")
		text = text.replace("<", "&lt;")
		p = XPreformatted("<b>" + text + "</b>", self.codeStyle)
		#p = PythonPreformatted(text, self.codeStyle)

		self.stories.append(p)
		self.stories.append(Spacer(1,0.2*inch))


