#!/usr/bin/env python2.2
import unittest
from Kplot import *
from Kplot.tools import abstract
import Kplot
import Numeric
import sys

def alwayswhite(value):
	return Color.white

class DummyObject(Kplot.plotobject.PlotObject):
	def __init__(self):
		Kplot.plotobject.PlotObject.__init__(self, {})
	def plot(self, dev):
		pass

class DeviceTest(unittest.TestCase):
	def createDevice(self):
		abstract()

	def setUp(self):
		#if not hasattr(self, "dev"):
		self.dev = self.createDevice()

	def tearDown(self):
		pass

	def testLinestyles(self):
		linestyles = self.dev.getLinestyles()
		self.assert_("noline" in linestyles, "device should support linestyle 'noline'")
		self.assert_("normal" in linestyles, "device should support linestyle 'normal'")
		self.assert_("dashed" in linestyles, "device should support linestyle 'dashed'")
		self.assert_("dotted" in linestyles, "device should support linestyle 'dotted'")

	def testFillstyles(self):
		fillstyles = self.dev.getFillstyles()
		self.assert_("nofill" in fillstyles, "device should support fillstyle 'nofill'")
		self.assert_("solid" in fillstyles, "device should support fillstyle 'solid'")
		self.assert_("hatched" in fillstyles, "device should support fillstyle 'hatched'")
		self.assert_("crosshatched" in fillstyles, "device should support fillstyle 'crosshatched'")

	def testFonts(self):
		fonts = self.dev.getFonts()
		self.assert_("normal" in fonts, "device should support font 'normal'")
		self.assert_("roman" in fonts, "device should support font 'roman'")
		self.assert_("italic" in fonts, "device should support font 'italic'")

	def testMatrix(self):

		m1 = Kplot.Matrix.scaleXY(10, 20)
		m2 = Kplot.Matrix.shear(2,3)
		self.assertEquals(self.dev.getWorldMatrix(), Matrix(), "world matrix has no identity matrix at top")
		self.dev.pushWorldMatrix(m1)
		self.dev.pushWorldMatrix(m2)
		self.assertEquals(self.dev.getWorldMatrix(), m1 * m2, "world matrix stack is invalid")

		self.assertEquals(self.dev.getLastWorldMatrix(), m2, "world matrix stack is invalid")
		self.dev.popWorldMatrix()
		self.assertEquals(self.dev.getWorldMatrix(), m1, "world matrix stack is invalid")
		self.dev.popWorldMatrix()
		self.assertEquals(self.dev.getWorldMatrix(), Matrix(), "world matrix has no identity matrix at top")

		m1 = Kplot.Matrix.scaleXY(10, 20)
		m2 = Kplot.Matrix.shear(2,3)
		self.assertEquals(self.dev.getViewportMatrix(), Matrix(), "viewport matrix has no identity matrix at top")
		self.dev.pushViewportMatrix(m1)
		self.dev.pushViewportMatrix(m2)
		self.assertEquals(self.dev.getViewportMatrix(), m1 * m2, "viewport matrix stack is invalid")

		self.assertEquals(self.dev.getLastViewportMatrix(), m2, "viewport matrix stack is invalid")
		self.dev.popViewportMatrix()
		self.assertEquals(self.dev.getViewportMatrix(), m1, "viewport matrix stack is invalid")
		self.dev.popViewportMatrix()
		self.assertEquals(self.dev.getViewportMatrix(), Matrix(), "viewport matrix has no identity matrix at top")

	def testPlotPolyline(self):
		x = [1,2,3]
		y = [2,3,4,5]
		linestyles = self.dev.getLinestyles()
		for linestyle in linestyles:
			self.dev.plotPolyline(x, y, linewidth=1, linestyle=linestyle, color="green", units="mm")

		x = Numeric.array([1,2,3])
		y = Numeric.array([2,3,4,5])
		self.dev.plotPolyline(x, y, linewidth=1, linestyle="normal", color="green", units="mm")

		x = (1,2,3)
		y = (2,3,4,5)
		self.dev.plotPolyline(x, y, linewidth=1, linestyle="normal", color="green", units="mm")

	def testPlotLines(self):
		x = [1,2,3,6]
		y = [2,3,4,5]
		linestyles = self.dev.getLinestyles()
		for linestyle in linestyles:
			self.dev.plotLines(x, y, linewidth=1, linestyle=linestyle, color="green", units="mm")

	def testPlotPolygon(self):
		x = [1,2,3]
		y = [2,3,4,5]
		linestyles = self.dev.getLinestyles()
		fillstyles = self.dev.getFillstyles()
		for fillstyle in fillstyles:
			for linestyle in linestyles:
				self.dev.plotPolygon(x, y, linewidth=1, linestyle=linestyle, fillstyle=fillstyle, color="green", units="mm")

		x = Numeric.array([1,2,3])
		y = Numeric.array([2,3,4,5])
		self.dev.plotPolygon(x, y, linewidth=1, linestyle="normal", fillstyle="solid", color="green", units="mm")

		x = (1,2,3)
		y = (2,3,4,5)
		self.dev.plotPolygon(x, y, linewidth=1, linestyle="normal", fillstyle="solid", color="green", units="mm")


	def testPlotText(self):
		fonts = self.dev.getFonts()
		for font in fonts:
			self.dev.plotText("", (0,0), valign="center", halign="center", font=font, fontsize=20, weight=12, color="blue", bgcolor="red", textangle=0, units="mm")

		for halign in ["left", "right", "center"]:
			for valign in ["top", "bottom", "center"]:
				self.dev.plotText("test", (0,0), valign=valign, halign=halign, font="normal",
					fontsize=20, weight=12, color="blue", bgcolor="red", textangle=0, units="mm")


	def testPlotImage(self):
		colormap = ColorMap.rainbow
		data = [[1,2],[2,3]]
		datamin, datamax = 1,3
		function = lambda x: x * 0.5 + 0.5
		self.dev.plotImage(data, colormap=colormap, datamin=datamin, datamax=datamax, function=function)

		function = lambda x: x * 2
		self.dev.plotImage(data, colormap=colormap, datamin=datamin, datamax=datamax, function=function)

		datamin, datamax = 3,1
		self.dev.plotImage(data, colormap=colormap, datamin=datamin, datamax=datamax, function=function)

	def testCalls(self):
		clipping = self.dev.getClipping()
		self.dev.setClipping(not clipping)
		self.assertEqual(clipping, not self.dev.getClipping())
		self.assertEqual(clipping, not self.dev.getClipping())

		x = Numeric.array([1,2,3])
		y = Numeric.array([2,3,4,5])

		clipping = 1
		self.dev.setClipping(clipping)
		self.assertEqual(clipping, self.dev.getClipping())
		self.dev.plotPolyline(x, y, linewidth=1, linestyle="normal", color="green", units="mm")
		self.assertEqual(clipping, self.dev.getClipping())

		clipping = 0
		self.dev.setClipping(clipping)
		self.assertEqual(clipping, self.dev.getClipping())
		self.dev.plotPolyline(x, y, linewidth=1, linestyle="normal", color="green", units="mm")
		self.assertEqual(clipping, self.dev.getClipping())

		panel = self.dev.createPanel()
		panel.add(DummyObject())
		#stdout = sys.stdout
		#sys.stdout = "dsa"
		self.dev.printMembers()
		#sys.stdout = stdout

		self.dev.clear()
		self.dev.output()

	def testUnits(self):
		self.assertEqual(self.dev.getUnitMatrix(), self.dev.getUnitMatrix("mm"))
		self.assertEqual(self.dev.getDimension(), self.dev.getDimension("mm"))

		for units in ["cm", "mm", "inch", "points", "normalised"]:
			self.dev.setDefaultUnits(units)
			self.assertEqual(self.dev.getDimension(), self.dev.getDimension(units))
			self.assertEqual(self.dev.getUnitMatrix(), self.dev.getUnitMatrix(units))

		self.dev.setDefaultUnits("mm")

	def testCloseAndOpen(self):
		self.dev.close()
		self.dev.open()
		self.dev.close()
		self.dev.open()
		x = [1,2,3]
		y = [2,3,4,5]
		linestyle = "dotted"
		fillstyle = "solid"
		self.dev.plotPolygon(x, y, linewidth=1, linestyle=linestyle, fillstyle=fillstyle, color="green", units="mm")

	def testChangeSize(self):
		self.dev.changeSize((0.5, 0.5), "normalised")
		x, y = self.dev.getDimension("normalised")
		self.assert_(abs(x - 1) < 1e-10)
		self.assert_(abs(y - 1) < 1e-10)

		self.dev.changeSize((100, 200), "mm")
		x, y = self.dev.getDimension("mm")
		self.assert_(abs(x - 100) < 1e-4)
		self.assert_(abs(y - 200) < 1e-4)

		self.dev.changeSize((10, 20), "cm")
		x, y = self.dev.getDimension("cm")
		self.assert_(abs(x - 10) < 1e-4)
		self.assert_(abs(y - 20) < 1e-4)

		self.dev.changeSize((10, 20), "inch")
		x, y = self.dev.getDimension("inch")
		self.assert_(abs(x - 10) < 1e-4)
		self.assert_(abs(y - 20) < 1e-4)

		self.dev.changeSize((100, 200), "points")
		x, y = self.dev.getDimension("points")
		self.assert_(abs(x - 100) < 1e-4)
		self.assert_(abs(y - 200) < 1e-4)

	def testHardcopy(self):
		panel = self.dev.createPanel()
		panel.add(Kplot.Line(p1=(0,0), p2=(1,1)))
		p2 = panel.createPanel()
		self.dev.hardcopy()


	def testColors(self):
		x = [1,2,3]
		y = [2,3,4,5]
		colors = ["fg", "foreground", "bg", "background", "blue"]

		for color in colors:
			self.dev.plotPolyline(x, y, linewidth=1, linestyle="normal", color=color, units="mm")

		self.dev.setBgColor("red");
		self.dev.setFgColor("green");
