#!/usr/bin/env python2.2

from Kplot import *
import unittest
import math
import Numeric


class MatrixTest(unittest.TestCase):

	def testCtor(self):
		m1 = Matrix([[1,2,3], [1,2,3], [1,2,3]])
		m2 = Matrix([[1,2,3], [1,2,3], [1,2,3]])
		m3 = Matrix.shear(2,3) * Matrix.rotate(1.4)
		m4 = Matrix(m3)
		self.assertEqual(m1, m2)
		self.assertEqual(m3, m4)

	def testIdentity(self):
		m1 = Matrix()
		m2 = Matrix()
		self.assertEqual(m1, m2)
		self.assertEqual(m1.inverse(), m2)
		self.assertEqual(m1, m2.inverse())

	def testRotate(self):
		m1 = Matrix.rotate(45.0 * math.pi/180)
		m2 = Matrix.rotate(-45.0 * math.pi/180)
		self.assertEqual(m1.inverse(), m2)
		self.assertEqual(m1, m2.inverse())
		p1 = [1, 0]
		p2 = [1/math.sqrt(2), 1/math.sqrt(2)]
		self.assertEqual(m1 * p1, Vector(p2))

	def testTranslate(self):
		p1 = (10.0, 20.0)
		p2 = (-10.0, -20.0)
		m1 = Matrix.translate(p1)
		m2 = Matrix.translate(p2)
		self.assertEqual(m1.inverse(), m2)
		self.assertEqual(m1, m2.inverse())
		p1 = [1, 2]
		p2 = [11, 22]
		self.assertEqual(m1 * p1, Vector(p2))

	def testScale(self):
		s1 = 20.0
		s2 = 1 / s1
		m1 = Matrix.scale(s1)
		m2 = Matrix.scale(s2)
		self.assertEqual(m1.inverse(), m2)
		self.assertEqual(m1, m2.inverse())
		p1 = [1, 2]
		p2 = [1*s1, 2*s1]
		self.assertEqual(m1 * p1, Vector(p2))

	def testScaleXY(self):
		x1, y1 = 20.0, 30.0
		x2, y2 = 1/x1,  1/y1
		m1 = Matrix.scaleXY(x1, y1)
		m2 = Matrix.scaleXY(x2, y2)
		self.assertEqual(m1.inverse(), m2)
		self.assertEqual(m1, m2.inverse())
		p1 = [1, 2]
		p2 = [1*x1, 2*y1]
		self.assertEqual(m1 * p1, Vector(p2))

	def testShear(self):
		sx1, sy1 = 20.0, 30.0
		sx2, sy2 = 1/sx1,  1/sy1
		m1 = Matrix.shear(sx1, sy1)
		m2 = Matrix.shear(-sx1, -sy1)
		#self.assertEqual(m1.inverse(), m2)
		#self.assertEqual(m1, m2.inverse())
		p1 = (1, 2)
		p2 = (1+2*sx1, 2+1*sy1)
		self.assertEqual(m1 * p1, Vector(p2))

	def testMatrix(self):
		m = Matrix()
		m *= Matrix.translate((10, 5))
		m *= Matrix.rotate(180*math.pi/180.0)
		m *= Matrix.scale(0.5)

		m = Matrix()
		m *= Matrix.translate((1, 1))

		x = [1,2,3,4,5]
		y = [10,20,30,40,50]
		x,y = m.mulXY(x, y)

		x = (100,2,3,4,5)
		y = (10,20,30,40,50)
		x,y = m.mulXY(x, y)

		x = Numeric.array([1,2,3,4,5])
		y = Numeric.array([10,20,30,40,50])
		x,y = m.mulXY(x, y)

	def testLocation(self):
		p = Vector(10, 5)
		m1 = Matrix.translate(p) * Matrix.shear(4,1) * Matrix.rotate(3.1)
		m2 = Matrix.shear(4,1) * Matrix.rotate(3.1)
		self.assertEquals(m1.location(), p)
		self.assertEquals(m2, m1.nolocation())


	def testSequence(self):
		m1 = Matrix()
		m2 = Matrix.translate((10, 20))
		m1[0][2] = 10
		m1[1][2] = 20
		self.assertEquals(m1, m2)
		self.assertEquals(m2[0][2], 10)
		self.assertEquals(m2[1][2], 20)


#def suite():
#	suite = unittest.TestSuite()
#	suite.addTest(unittest.makeSuite(MatrixTest))
#	return suite

if __name__ == '__main__':
	unittest.main()


