#!/usr/bin/env python2.2
from Kplot import *
import unittest
import math
from math import *

class VectorTest(unittest.TestCase):
	def setUp(self):
		pass

	def testCtor(self):
		v1 = Vector(10, 20)
		v2 = Vector([10, 20])
		v3 = Vector((10, 20))
		v4 = Vector(v1)
		v5 = Vector(v1.x, v2.y)
		self.assertEqual(v1.x, 10)
		self.assertEqual(v2.x, 10)
		self.assertEqual(v3.x, 10)
		self.assertEqual(v4.x, 10)
		self.assertEqual(v5.x, 10)
		self.assertEqual(v1.y, 20)
		self.assertEqual(v2.y, 20)
		self.assertEqual(v3.y, 20)
		self.assertEqual(v4.y, 20)
		self.assertEqual(v5.y, 20)

	def testScale(self):
		v1 = Vector(10, 10)
		v1 = v1.scale(0.1)
		self.assertEqual(v1, Vector(1,1))

	def testLength(self):
		x = 10
		y = 20
		length = math.sqrt(x*x+y*y)
		v = Vector(x, y)
		self.assertEqual(v.length(), length)

	def testUnit(self):
		n = 1/math.sqrt(2.0)
		v1 = Vector(n,n)
		self.assertEquals(v1, Vector(10, 10).unit())
		self.assert_(abs(v1.length() - 1) < 1e-10)

	def testDot(self):
		v1 = Vector(1,0)
		v2 = Vector(0,1)
		theta = v1.dot(v2)
		self.assert_(abs(theta) < 1e-10)

		v1 = Vector(1,1).unit()
		v2 = Vector(0,1)
		theta = v1.dot(v2)
		angle = acos(theta) * 180/pi
		self.assert_(abs(angle-45) < 1e-10)

	def testAngle(self):
		v1 = Vector(1,0)
		v2 = Vector(1,1)
		v3 = Vector(0,1)
		v4 = Vector(-1,-1)
		self.assertEquals(v1.angle(), 0)
		self.assertEquals(v2.angle(), pi/4)
		self.assertEquals(v3.angle(), pi/2)
		self.assertEquals(v4.angle(), pi+pi/4)

	def testAdd(self):
		x1, y1 = (20, 30)
		x2, y2 = (50, 80)
		x3, y3 = (x1+x2, y1+y2)
		v1 = Vector((x1, y1))
		v2 = Vector((x2, y2))
		v3 = Vector((x3, y3))
		self.assertEqual(v1+v2, v3)

	def testSub(self):
		x1, y1 = (20, 30)
		x2, y2 = (50, 80)
		x3, y3 = (x1-x2, y1-y2)
		v1 = Vector((x1, y1))
		v2 = Vector((x2, y2))
		v3 = Vector((x3, y3))
		self.assertEqual(v1-v2, v3)

	def testNeg(self):
		v1 = Vector(5, 6)
		v2 = Vector(-5, -6)
		self.assertEquals(-v1, v2)
		self.assertEquals(v1, -v2)


	def testItem(self):
		v1 = Vector(10, 20)
		v1.x += 30
		v1[0] -= 30
		self.assert_(v1.x == 10)
		self.assert_(v1[0] == 10)
		v1.y += 50
		v1[1] -= 50
		self.assert_(v1.y == 20)
		self.assert_(v1[1] == 20)

	def testEquals(self):
		v1 = Vector(10, 20)
		v2 = Vector(10, 20)
		v3 = Vector(20, 30)
		self.assertEquals(v1, v2)
		self.assertNotEquals(v1, v3)
		self.assert_(v1 == v2)
		self.assert_(v1 != v3)

	def testCalls(self):
		v1 = Vector(1,1)
		v2 = Vector(1,1)
		v1.__repr__()
		v1.__ne__(v2)
		v1.__eq__(v2)

if __name__ == '__main__':
	unittest.main()


